/*
This is a Wav player for Grannies push button sound player.
If Grannie pushes a button a clear voice command is played.
This is helpful in using the Google Home for specific calls

Kent Peterson 15 Dec 2017
Code based on Adafruit Wave Shield
*/

#include <FatReader.h>
#include <SdReader.h>
#include <avr/pgmspace.h>
#include "WaveUtil.h"
#include "WaveHC.h"

SdReader card;    // This object holds the information for the card
FatVolume vol;    // This holds the information for the partition on the card
FatReader root;   // This holds the information for the filesystem on the card
FatReader f;      // This holds the information for the file we're playing

const int amp =  6;      // the number of the amp enable pin

WaveHC wave;      // This is the only wave (audio) object, since we will only play one at a time

#define DEBOUNCE 100  // button debouncer

// this handy function will return the number of bytes currently free in RAM, great for debugging!   
int freeRam(void)
{
  extern int  __bss_end; 
  extern int  *__brkval; 
  int free_memory; 
  if((int)__brkval == 0) {
    free_memory = ((int)&free_memory) - ((int)&__bss_end); 
  }
  else {
    free_memory = ((int)&free_memory) - ((int)__brkval); 
  }
  return free_memory; 
} 
 
void sdErrorCheck(void)
{
  if (!card.errorCode()) return;
  putstring("\n\rSD I/O error: ");
  Serial.print(card.errorCode(), HEX);
  putstring(", ");
  Serial.println(card.errorData(), HEX);
  while(1);
}

/*
First thing that must be done is initializing the SD card for reading. 
This is a multistep process. In the setup loop you can see the multiple 
checks we do as we proceed through the initialization process. 
*/

void setup() {
  // set up serial port and print
  Serial.begin(9600);
  putstring_nl("WaveHC with 6 buttons");
  // Check how much free RAM we have
  putstring("Free RAM: ");       // This can help with debugging, running out of RAM is bad
  Serial.println(freeRam());     // if this is under 150 bytes it may spell trouble!
  
  // Set the output pins for the DAC control. This pins are defined in the library
  pinMode(2, OUTPUT);
  pinMode(3, OUTPUT);
  pinMode(4, OUTPUT);
  pinMode(5, OUTPUT);
 
  // pin13 LED
  pinMode(13, OUTPUT);

  // pin6 is audio amp enanble
  pinMode(6, OUTPUT);

  // enable pull-up resistors on switch pins (analog inputs)
  digitalWrite(14, HIGH);		// Analog 0, Button 1
  digitalWrite(15, HIGH);		// Analog 1, Button 2
  digitalWrite(16, HIGH);		// Analog 2, Button 3
  digitalWrite(17, HIGH);		// Analog 3, Button 4
  digitalWrite(18, HIGH);		// Analog 4, Button 5
  digitalWrite(19, HIGH);		// Analog 5, Button 6
  
  // Initialize SD card to see if it responds, halt if it fails
  //  if (!card.init(true)) { //play with 4 MHz spi if 8MHz isn't working for you
  if (!card.init()) {         //play with 8 MHz spi (default faster!)  
    putstring_nl("Card init. failed!");  // Something went wrong, lets print out why
    sdErrorCheck();
    while(1);                            // then 'halt' - do nothing!
  }
  // Allow partial block reads
  // enable optimize read - some cards may timeout. Disable if you're having problems
  card.partialBlockRead(true);
 
// Now we will look for a FAT partition! SD card mush have one.
  uint8_t part;
  for (part = 0; part < 5; part++) {     // we have up to 5 slots to look in
    if (vol.init(card, part)) 
      break;                             // we found one, lets bail
  }
  if (part == 5) {                       // if we ended up not finding one  :(
    putstring_nl("No valid FAT partition!");
    sdErrorCheck();      // Something went wrong, lets print out why
    while(1);                            // then 'halt' - do nothing!
  }
  
  // Lets tell the user about what we found
  putstring("Using partition ");
  Serial.print(part, DEC);
  putstring(", type is FAT");
  Serial.println(vol.fatType(),DEC);     // FAT16 or FAT32?
  
  // Try to open the root directory
  if (!root.openRoot(vol)) {
    putstring_nl("Can't open root dir!"); // Something went wrong,
    while(1);                             // then 'halt' - do nothing!
  }
  
  // Whew! We got past the tough parts.
  putstring_nl("Ready!");
}

/*
Button interfacing
We want to play a sound each time a button is pressed. We will use a function called 
check_switches() that goes through the 6 buttons (digital 14 through 20) to see if 
they have been pressed. If so, we play SOUND1.WAV (for example) completely through.
 The function that we call here that does the playing is called playcomplete() and 
 we pass the name of the Wave file in quotes just like you see here.
*/

void loop() {
  //putstring(".");            // uncomment this to see if the loop isnt running
  switch (check_switches()) {
    case 1:
      putstring_nl("Button 1");
      playcomplete("SOUND1.WAV"); // WAV file for button 1
      break;
    case 2:
      putstring_nl("Button 2");
      playcomplete("SOUND2.WAV"); // WAV file for button 2
      break;
    case 3:
      putstring_nl("Button 3");
      playcomplete("SOUND3.WAV"); // WAV file for button 3
      break;
    case 4:
      putstring_nl("Button 4");
      playcomplete("SOUND4.WAV"); // WAV file for button 4
      break;
    case 5:
      putstring_nl("Button 5");
      playcomplete("SOUND5.WAV"); // WAV file for button 5
      break;
    case 6:
      putstring_nl("Button 6");
      playcomplete("SOUND6.WAV"); // WAV file for button 6
  }
}

byte check_switches()
{
  static byte previous[6];
  static long time[6];
  byte reading;
  byte pressed;
  byte index;
  pressed = 0;

  for (byte index = 0; index < 6; ++index) {
    reading = digitalRead(14 + index);
    if (reading == LOW && previous[index] == HIGH && millis() - time[index] > DEBOUNCE)
    {
      // switch pressed
      time[index] = millis();
      pressed = index + 1;
      break;
    }
    previous[index] = reading;
  }
  // return switch number (1 - 6)
  return (pressed);
}

/*
Playcomplete & Playfile
Here is where we open the file and play it. 

Playcomplete is very simple, it just calls a function that starts the audio 
playback and then sits in a loop doing nothing. 

Playfile is the important function. It finds and opens the file and plays it. 

1) It first sees if we're already playing any audio. If so, it stops it.
2) Now it opens the root directory and looks for the file by the name we 
requested. If it can't find it, the function returns.
3) If it finds it, it tries to turn it into a Wave file object, looking for 
the right header in the file. If not it also returns.
4) If it succeeds, it begins to play.
*/

// Plays a full file from beginning to end with no pause.
void playcomplete(char *name) {
  // call our helper to find and play this name

    Serial.print("Play WAV file!! ");   // serial monitor print to track progress
    Serial.println("\t");
    delay(100);
    Serial.print("Enable Amp");   // serial monitor print to track progress
    Serial.println("\t");
    digitalWrite(amp, HIGH);  //Enable the audio amp
  
  playfile(name);

  
  while (wave.isplaying) {
  // do nothing while its playing
  }
  // now its done playing

    Serial.print("Disable Amp");   // serial monitor print to track progress
    Serial.println("\t");
    digitalWrite(amp, LOW);  //Disable the audio amp
  
}

void playfile(char *name) {
  // see if the wave object is currently doing something
  if (wave.isplaying) {// already playing something, so stop it!
    wave.stop(); // stop it
  }
  // look in the root directory and open the file
  if (!f.open(root, name)) {
    putstring("Couldn't open file "); Serial.print(name); return;
  }
  // OK read the file and turn it into a wave object
  if (!wave.create(f)) {
    putstring_nl("Not a valid WAV"); return;
  }
  
  // ok time to play! start playback
  wave.play();
}
